@@ -1,8 +1,8 @@
-# -*- coding: utf-8 -*-
 import random
 import time
 
 import pytest
+
 from flask_caching import Cache
 
 try:
@@ -19,6 +19,11 @@ def test_cache_set(app, cache):
     assert cache.get("hi") == "hello"
 
 
+def test_cache_has(app, cache):
+    cache.add("hi", "hello")
+    assert cache.has("hi")
+
+
 def test_cache_add(app, cache):
     cache.add("hi", "hello")
     assert cache.get("hi") == "hello"
@@ -308,3 +313,25 @@ def test_cache_forced_update_params(app, cache):
         # this time the forced_update should have returned True, so
         # cached_function should have been called again
         assert cached_call_counter[1] == 2
+
+
+def test_generator(app, cache):
+    """ test function return generator"""
+    with app.test_request_context():
+
+        @cache.cached()
+        def gen():
+            return (str(time.time()) for i in range(2))
+
+        time_str = gen()
+        time.sleep(1)
+        assert gen() == time_str
+
+        @cache.cached()
+        def gen_yield():
+            yield str(time.time())
+            yield str(time.time())
+
+        time_str = gen_yield()
+        time.sleep(1)
+        assert gen_yield() == time_str
